﻿namespace IC_DB
{
   /// <summary>
   /// Daten für einen IC-Pin.
   /// </summary>
   public class PinDefinition
   {
      /// <summary>1-basierte Nummer des Pins im IC. 0 bei undefiniertem Pin.</summary>
      public int Number = 0;

      /// <summary>Typ des Pins.</summary>
      public PinType Type = PinType.NC;

      /// <summary>Bezeichnung des Pins.</summary>
      public String Name = "";

      /// <summary>
      /// Initialisiert eine neue Instanz der <see cref="PinDefinition"/>-Klasse.
      /// </summary>
      public PinDefinition() { }


      /// <summary>
      /// Liefert eine tiefe Kopie des Objekts.
      /// </summary>
      /// <returns>Eine tiefe Kopie des Objekts.</returns>
      public PinDefinition DeepCopy()
      {
         PinDefinition other = (PinDefinition)this.MemberwiseClone(); // Die elementaren Felder
         other.Name = Name.DeepCopy();
         return other;
      }

      /// <summary>
      /// Liefert eine textliche Darstellung des Objekts.
      /// </summary>
      /// <returns>Eine textliche Darstellung des Objekts.</returns>
      public override string ToString()
      {
         return Number + ": " + Name + " (" + Type.ToString() + ")";
      }

      /// <summary>
      /// Vergleicht, ob zwei Instanzen von <see cref="PinDefinition"/> den gleichen Inhalt haben.
      /// </summary>
      /// <param name="obj1">Erstes Vergleichsobjekt.</param>
      /// <param name="obj2">Zweites Vergleichsobjekt.</param>
      /// <returns><b>true</b>, wenn die Inhalte beider Instanzen gleich sind, ansonsten <b>false</b>.</returns>
      /// <remarks>Feld <see cref="Number"/> wird nicht verglichen, maschinell generiert.</remarks>
      public static bool operator ==(PinDefinition obj1, PinDefinition obj2)
      {
         if (obj1.Name != obj2.Name) return false;
         if (obj1.Type != obj2.Type) return false;
         return true;
      }

      /// <summary>
      /// Vergleicht, ob zwei Instanzen von <see cref="PinDefinition"/> nicht den gleichen Inhalt haben.
      /// </summary>
      /// <param name="obj1">Erstes Vergleichsobjekt.</param>
      /// <param name="obj2">Zweites Vergleichsobjekt.</param>
      /// <returns><b>true</b>, wenn die Inhalte beider Instanzen nicht gleich sind, ansonsten <b>false</b>.</returns> 
      /// <remarks>Feld <see cref="Number"/> wird nicht verglichen, maschinell generiert.</remarks>
      public static bool operator !=(PinDefinition obj1, PinDefinition obj2)
      {
         if (obj1.Name != obj2.Name) return true;
         if (obj1.Type != obj2.Type) return true;
         return false;
      }

      /// <summary>
      /// Bestimmt, ob der aktuelle <see cref="PinDefinition"/> mit dem angegebenen Object übereinstimmt.
      /// </summary>
      /// <param name="obj">Das Vergleichsobjekt.</param>
      /// <returns><b>true</b>, wenn <paramref name="obj"/> vom Typ <see cref="TestStep"/> ist 
      /// und gleich dem aktuellen <see cref="TestStep"/> ist.</returns>
      public override bool Equals(object? obj)
      {
         if (ReferenceEquals(this, obj))
         {
            return true;
         }

         if (ReferenceEquals(obj, null))
         {
            return false;
         }

         return this == (PinDefinition)obj;
      }

      /// <summary>
      /// Fungiert als Standardhashfunktion.
      /// </summary>
      /// <returns>Ein Hashcode für das aktuelle Objekt.</returns>
      public override int GetHashCode()
      {
         return Type.GetHashCode();
      }
   }
}
