﻿using IC_DB;
using static IC_Tester_Controls.IcSelector;

namespace IC_Tester_Controls
{
   /// <summary>
   /// Steuerelement zur Anzeige der Liste mit den erfassten ICs und Auswahl der Bearbeitung.
   /// </summary>
   public partial class IcSelector : UserControl
   {
      /// <summary>Aufzählung der möglichen Aktionen.</summary>
      public enum IcSelectorAction
      {
         /// <summary><see cref="IcDefinition"/>-Objekt anzeigen.</summary>
         Show,
         /// <summary>Neues <see cref="IcDefinition"/>-Objekt anlegen.</summary>
         Add,
         /// <summary><see cref="IcDefinition"/>-Objekt bearbeiten.</summary>
         Edit,
         /// <summary><see cref="IcDefinition"/>-Objekt Duplizieren.</summary>
         Duplicate,
         /// <summary><see cref="IcDefinition"/>-Objekt löschen.</summary>
         Remove,
         /// <summary>Test für <see cref="IcDefinition"/>-Objekt durchführen.</summary>
         Test,
         /// <summary>XML-Serialisation des <see cref="IcDefinition"/>-Objekts in eine Datei speichern.</summary>
         Export,
         /// <summary><see cref="IcDefinition"/>-Objekts aus einer XML-Datei einlesen.</summary>
         Import
      }

      /// <summary>
      /// Stellt die Methode dar, die das <see cref="ActionSelected"/>-Ereignis der <see cref="IcSelector"/>-Klasse behandelt.
      /// </summary>
      /// <param name="sender">Die Quelle des Ereignisses.</param>
      /// <param name="e">Ein <see cref="IcSelectorActionEventArgs"/>-Objekt, das die Ereignisdaten enthält.</param>
      public delegate void ActionEventHandler(object sender, IcSelectorActionEventArgs e);
      /// <summary>Tritt ein, wenn ein Item des Menüs zur Auswahl der Bearbeitung angklickt wurde.</summary>
      public event ActionEventHandler? ActionSelected;

      /// <summary>
      /// Initialisiert eine neue Instanz der <see cref="IcSelector"/>-Klasse.
      /// </summary>
      public IcSelector() { InitializeComponent(); }

      /// <summary>
      /// Fügt ein neues <see cref="IcDefinition"/>-Objekt an.
      /// </summary>
      /// <param name="Ic">Das anzufügende <see cref="IcDefinition"/>-Objekt.</param>
      /// <param name="SortList"><b>true</b>: die Liste wird nach dem Einfügen sortiert.</param>
      /// <returns>Der nullbasierte Index des Elements in der Auflistung.</returns>
      public int add(IcDefinition Ic, bool SortList = true)
      {
         var ind = listBox.Items.Add(Ic);
         listBox.Sorted = false;
         listBox.Sorted = true;
         return ind;
      }
      /// <summary>
      /// Löscht das angegebene <see cref="IcDefinition"/>-Objekt aus der Liste.
      /// </summary>
      /// <param name="Ic">Das zu löschende <see cref="IcDefinition"/>-Objekt.</param>
      public void remove(IcDefinition Ic) { listBox.Items.Remove(Ic); }
      /// <summary>
      /// Ersetzt das aktuell selektierte <see cref="IcDefinition"/>-Objekt.
      /// </summary>
      /// <param name="Ic">Das neue <see cref="IcDefinition"/>-Objekt.</param>
      public void Replace(IcDefinition Ic)
      {
         for (int i = 0; i < listBox.Items.Count; i++)
         {
            if (Ic.InternalID == ((IcDefinition)listBox.Items[i]).InternalID)
            {
               listBox.Items[i] = Ic;
               listBox.Sorted = false;
               listBox.Sorted = true;
               return;
            }
         }
      }

      /// <summary>
      /// Sortiert die Liste.
      /// </summary>
      public void Sort()
      {
         listBox.Sorted = false;
         listBox.Sorted = true;
      }

     /// <summary>
     /// Selektiert das angegebene IC in der Listbox.
     /// </summary>
     /// <param name="ic">Das zu selektierende IC. <b>null</b> für keine Selection.</param>
     public void SelectIC(IcDefinition? ic)
      {
         if (ic==null)
         { 
            listBox.SelectedIndex = -1;
            return;
         }
         for (int i = 0; i < listBox.Items.Count; i++)
         {
            if (((IcDefinition)listBox.Items[i]).InternalID == ic.InternalID)
               listBox.SelectedIndex =i;
         }
      }

      /// <summary>
      /// Ruft den Index des ersten sichtbaren Elements ab oder legt diesen fest.
      /// </summary>
      public int TopIndex
      {
         get { return listBox.TopIndex; }
         set { listBox.TopIndex = value; }
      }


      private void listBox_MouseDown(object sender, MouseEventArgs e)
      {
         if (e.Button == MouseButtons.Right)
         {
            listBox.SelectedIndex = listBox.IndexFromPoint(e.X, e.Y);
         }
      }

      private void contextMenu_Opening(object sender, System.ComponentModel.CancelEventArgs e)
      {
         if (listBox.SelectedIndex > -1)
         {
            miAdd.Visible = true;
            miEdit.Visible = true;
            miRemove.Visible = true;
            miTest.Visible = true;
            miDuplicate.Visible = true;
            toolStripSeparator1.Visible = true;
            miExport.Visible = true;
         }
         else
         {
            miAdd.Visible = true;
            miEdit.Visible = false;
            miRemove.Visible = false;
            miTest.Visible = false;
            miDuplicate.Visible = false;
            toolStripSeparator1.Visible = false;
            miExport.Visible = false;
         }

      }

      private void listBox_SelectedIndexChanged(object sender, EventArgs e)
      {
         ActionSelected?.Invoke(this, new IcSelectorActionEventArgs(IcSelectorAction.Show, (IcDefinition)listBox.SelectedItem));
      }

      private void miAdd_Click(object sender, EventArgs e)
      {
         ActionSelected?.Invoke(this, new IcSelectorActionEventArgs(IcSelectorAction.Add, null));
      }

      private void miEdit_Click(object sender, EventArgs e)
      {
         ActionSelected?.Invoke(this, new IcSelectorActionEventArgs(IcSelectorAction.Edit, (IcDefinition)listBox.SelectedItem));
      }

      private void miDuplicate_Click(object sender, EventArgs e)
      {
         ActionSelected?.Invoke(this, new IcSelectorActionEventArgs(IcSelectorAction.Duplicate, (IcDefinition)listBox.SelectedItem));
      }

      private void miRemove_Click(object sender, EventArgs e)
      {
         ActionSelected?.Invoke(this, new IcSelectorActionEventArgs(IcSelectorAction.Remove, (IcDefinition)listBox.SelectedItem));
      }

      private void miTest_Click(object sender, EventArgs e)
      {
         ActionSelected?.Invoke(this, new IcSelectorActionEventArgs(IcSelectorAction.Test, (IcDefinition)listBox.SelectedItem));

      }

      private void miImport_Click(object sender, EventArgs e)
      {
         ActionSelected?.Invoke(this, new IcSelectorActionEventArgs(IcSelectorAction.Import, (IcDefinition)listBox.SelectedItem));
      }

      private void miExport_Click(object sender, EventArgs e)
      {
         ActionSelected?.Invoke(this, new IcSelectorActionEventArgs(IcSelectorAction.Export, (IcDefinition)listBox.SelectedItem));
      }

      private void listBox_DrawItem(object sender, DrawItemEventArgs e)
      {
         if (e.Index < 0)
            return;
         e.DrawBackground();
         e.DrawFocusRectangle();
         IcDefinition ic = (IcDefinition)listBox.Items[e.Index];

         var bounds = new Rectangle(e.Bounds.X, e.Bounds.Y, 70, e.Bounds.Height);
         e.Graphics.DrawString(ic.Name + "........................", listBox.Font, new SolidBrush(listBox.ForeColor), bounds);
         bounds.X = 71;
         bounds.Width = e.Bounds.Width - 71;
         e.Graphics.DrawString(ic.Description, listBox.Font, new SolidBrush(listBox.ForeColor), bounds);
      }
   }


   /// <summary>
   /// Argumente für das <see cref="IcSelector.ActionSelected"/>-Ereignis.
   /// </summary>
   public class IcSelectorActionEventArgs
   {
      /// <summary>Die Aktion, asugeführt werden soll.</summary>
      public IcSelectorAction Action;
      /// <summary>Das betroffene <see cref="IcDefinition"/>-Objekt. <b>null</b> bei <see cref="IcSelectorAction.Add"/>.</summary>
      public IcDefinition? Ic;

      /// <summary>
      /// Initialisiert eine neue Instanz der <see cref="IcSelectorActionEventArgs"/>-Klasse.
      /// </summary>
      /// <param name="action">Die Aktion, asugeführt werden soll.</param>
      /// <param name="ic">Das betroffene <see cref="IcDefinition"/>-Objekt. <b>null</b> bei <see cref="IcSelectorAction.Add"/>.</param>
      public IcSelectorActionEventArgs(IcSelectorAction action, IcDefinition? ic)
      {
         Action = action;
         Ic = ic;
      }
   }
}
